/* ----------- Moonquake database library routines ( header file ) -----------+
|                                                                             |
|                              "mqdb.h" ver 1.00                              |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This is a header of the moonquake database library routine (mqlib). To
 *  compile this, you need to prepare an ANSI C compiler. 
 */

/* ============================= Include files ============================= */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "mqstrs.h"
#include "mqerror.h"
#include "mqfunc.h"

/* =========================== Macro definitions =========================== */
#ifndef _MQDB_H
#define _MQDB_H

    /* definition for NON ANSI C header files */
#ifndef FILENAME_MAX
#define FILENAME_MAX    255
#endif

    /* current library version */
#define MQLIB_LIBTYPE     "MQ"  /* library type is MQ */
#define MQLIB_RELEASE     1     /* release version */
#define MQLIB_MAJOR       0     /* major version */
#define MQLIB_MINOR       0     /* minor version */

    /* boolean argument */
#define MQ_TRUE           1
#define MQ_FALSE          0

/* maximum length of data header */
#define MQCHANNELMAX    255  /* maximum number of channels */
#define MQFILENAMELEN    12  /* maximum MQDB file name length */
#define MQFILDESCMAX     16  /* maximum file descriptors */
#define MQHDRLENMAX    1024  /* maximum length of the header */
#define MQHDRLINEMAX     80  /* maximum length of the one line of the header */
#define MQLIBVERLEN       5  /* maximum length of string "LibVersion" */
#define MQPROGSIGNLEN    16  /* maximum length of string "ProgSign" */
#define MQTOPLINEMIN     12  /* minimum top line length */

/* ======================== Structure declarations ========================= */

/* 
 *  enum-type expression for header data ( except MqYesNo ).
 *    MQAM:      data access method.
 *    MQDF:      data format type.
 *    MQDT:      data type.
 *    MQFF:      MQDB file format.
 *    MQIO:      IO stream type.
 *    MqMode:    observation mode.
 *    MQRW:      read/write flag.
 *    MqStat:    access status.
 *    MqYesNo:   boolean variable.
 */

typedef enum { MQ_DIRECT,   /* direct access to data file */
               MQ_TMP,      /* creating temporary files */
               MQ_SHM,      /* using shared memory ( NOT YET IMPLEMENTED ) */
               MQ_EMS       /* using EMS memory ( NOT YET IMPLEMENTED ) */
              } MQAM;
typedef enum { MQ_OLD, MQ_NEW }            MQDF;
typedef enum { MQ_LP, MQ_SP, MQ_TIDAL }    MQDT;
typedef enum { MQ_FULLTEXT,  /* data part are written in ASCII chars */
	       MQ_COMPOSITE, /* data part are written in dumped binaries */
	       MQ_XDR        /* data part are written in XDR format */
	      } MQFT;
typedef enum { MQ_FILE, MQ_STDIN, MQ_STDOUT, MQ_STDERR }   MQIO;
typedef enum { MQ_PEAKED, MQ_FLAT }        MqMode;
typedef enum { MQ_READ, MQ_WRITE, MQ_UNKNOWN }      MQRW;
typedef enum { MQ_NOACCESS, MQ_SUCCESS, MQ_FAIL }   MqStat;
typedef enum { MQ_YES, MQ_NO }             MqYesNo;

/*
 *  structure about time. 
 */

typedef struct {
    unsigned short  year;     /* year */
    unsigned short  day;      /* day ( from 1 to 366 ) */
    unsigned short  hour;     /* hour ( from 0 to 23 ) */
    unsigned short  minute;   /* minute ( from 0 to 59 ) */
    unsigned short  second;   /* second ( from 0 to 59 ) */
    unsigned short  ms;       /* microsecond ( from 0 to 999 ) */
    }  MqTime;

/*
 * structure for values that will be different in channels
 */

typedef struct {
    unsigned short  channels;            /* number of data channels */
    double          data[MQCHANNELMAX];  /* individual data */
    }   MqAmp;

/*
 *  structure about data access. 
 */

typedef struct {
    MqYesNo  AveAmp;       /* average amplitude */
    MqYesNo  Channels;     /* number of channels */
    MqYesNo  DataCreDate;  /* data creation date */
    MqYesNo  DataFormat;   /* data format of original data file */
    MqYesNo  DataModDate;  /* data modification date */
    MqYesNo  DataType;     /* data type */
    MqYesNo  EndRecord;    /* end record number in the original file */
    MqYesNo  FileType;     /* MQDB file format */
    MqYesNo  MaxAmp;       /* maximum amplitude */
    MqYesNo  NumOfData;    /* number of data */
    MqYesNo  ObsMode;      /* observation mode ( PEAKED / FLAT ) */
    MqYesNo  OrgFileName;  /* original data file name */
    MqYesNo  SampRate;     /* sampling rate */
    MqYesNo  Station;      /* observed station */
    MqYesNo  StartRecord;  /* start record number in the original file */
    MqYesNo  StartTime;    /* start time of the data */
    MqYesNo  TapeNumber;   /* original tape sequence number */
    }    MqAccess;

/*
 *  Structure for library version storage.
 */
typedef struct {
    char  LibType[2];        /* Library type descriptor ( 2 chars ) */
    char  Release;           /* release number */
    char  Major;             /* major version */
    char  Minor;             /* minor version */
    }   MqLibVer;

typedef struct {
    MqLibVer    LibVer;                    /* library version */
    char        ProgSign[MQPROGSIGNLEN];   /* program signature */
    }   MqLibInfo;

/*
 *  Structure for storage of header data.
 */

typedef struct {
    MqAmp           AveAmp;         /* average amplitude */
    unsigned short  Channels;       /* data channel number */
    MQDF            DataFormat;     /* data format */
    MQDT            DataType;       /* data type */
    unsigned long   EndRecord;      /* end record number */
    MQFT            FileType;       /* MQDB file type */
    MqAmp           MaxAmp;         /* maximum amplitude */
    time_t          ModDate;        /* modified date */
    unsigned long   NumOfData;      /* Number of Data */
    MqMode          ObsMode;        /* observation mode */
    char            OrgFile[FILENAME_MAX];    
                                    /* original data file name */
    MqAmp           SampRate;       /* sampling rate */
    unsigned long   StartRecord;    /* start record number */
    MqTime          StartTime;      /* start time of the data */
    unsigned short  Station;        /* station number */
    unsigned short  TapeNumber;     /* data number */
    }    MqHeader;

/*
 *  Structure for file descriptors.
 */

typedef struct {
    MqYesNo   InUse;     /* whether this file descriptor is in use */
    MQAM      accmethod; /* data access method */
    MQIO      iostream;  /* which IO stream is used as input/output */
    MQRW      ReadWrite; /* whether access is reading or writeing */
    char     *filename;  /* real input/output file name */
    MqLibInfo libinfo;   /* library information */
    }    MqFile;

/* ========================= Function declarations ========================= */
    /* upper level interface functions */
extern void            MqCreate( char *, MqAccess *, MqHeader *, double * );
extern double *        MqRead( char *, MqAccess *, MqHeader * );
extern void            MqReadHeader( char *, MqAccess *, MqHeader * );
extern void            MqReadData( char *, MqAccess *, double * );
extern void            MqInit( char * );

    /* middle level interface functions */
extern char *          MqCreTmpHdrFileName( unsigned short );
extern MqAmp           MqGetAveAmp( char * );
extern unsigned short  MqGetChannels( char * );
extern void            MqGetData( FILE *, double *, MqHeader * );
extern MQDF            MqGetDataFormat( char * );
extern MQDT            MqGetDataType( char * );
extern unsigned long   MqGetEndRecord( char * );
extern MQFT            MqGetFileType( char * );
extern char *          MqGetHeader( FILE * );
extern MqAmp           MqGetMaxAmp( char * );
extern unsigned long   MqGetNumOfData( char * );
extern MqMode          MqGetObsMode( char * );
extern void            MqGetOrgFile( char *, char * );
extern MqAmp           MqGetSampRate( char * );
extern unsigned long   MqGetStartRecord( char * );
extern MqTime          MqGetStartTime( char * );
extern unsigned short  MqGetStation( char * );
extern unsigned short  MqGetTapeNumber( char * );
extern MqLibInfo       MqGetTopLine( FILE * );
extern char *          MqPutAveAmp( MqAmp );
extern char *          MqPutChannels( unsigned short );
extern char *          MqPutCreateDate( void );
extern void            MqPutData( FILE *, double *, MqHeader * );
extern char *          MqPutDataFormat( MQDF );
extern char *          MqPutDataType( MQDT );
extern char *          MqPutEndRecord( unsigned long );
extern char *          MqPutFileType( MQFT );
extern char *          MqPutMaxAmp( MqAmp );
extern char *          MqPutNumOfData( unsigned long );
extern char *          MqPutObsMode( MqMode );
extern char *          MqPutOrgFile( char * );
extern char *          MqPutSampRate( MqAmp );
extern char *          MqPutStartRecord( unsigned long );
extern char *          MqPutStartTime( MqTime );
extern char *          MqPutStation( unsigned short );
extern char *          MqPutTapeNumber( unsigned short );
extern void            MqPutTopLine( char * );

   /* low level interface functions */
extern char *          MqCatInfoData( const char *, const char * );
extern unsigned short  MqCountSeparator( unsigned short, char * );
extern void            MqConvCharDouble( char *, MqAmp * );
extern void            MqCutLine( char *, char *, long );
extern char *          MqReadTmpHdrFile( unsigned short );
extern long            MqSearchHeader( char *, char * );
extern void            MqSeparateInfoData( char *, char * );
extern MqTime          MqSeparateMqTime( char * );
extern short           MqSetFilDesc( char *, MQRW, MQIO );

    /* utility functions */
extern double          MqCalcAveAmp( double *, unsigned long, unsigned long );
extern double          MqCalcMaxAmp( double *, unsigned long, unsigned long );
extern char *          MqCreFileName( unsigned short, MqTime, MQDT );
extern void            MqCheckFilDesc( unsigned short, MQRW );
extern void            MqCheckLibVer( MqLibVer );
extern MqMode          MqCheckObsMode( MqTime, unsigned short );
extern double          MqCheckSampRate( MQDT );
extern void            MqCheckStation( unsigned short );
extern void            MqCheckTime( MqTime * );
extern double          MqDiffTime( MqTime, MqTime );
extern void            MqPerror( char * );
extern void            MqSetAccessNo( MqAccess * );
extern void            MqSetAccessYes( MqAccess * );
extern void            MqSetWarnLevel( unsigned short );

/* ===================== Static variable definitions ======================= */
MqLibInfo        MqLib;   /* current MQLIB information */

    /* header access status flags. */
struct {
    MqStat  AveAmp;       /* average amplitude */
    MqStat  Channels;     /* number of channels */
    MqStat  DataCreDate;  /* data creation date */
    MqStat  DataFormat;   /* data format of original data file */
    MqStat  DataModDate;  /* data modification date */
    MqStat  DataType;     /* data type */
    MqStat  EndRecord;    /* end record number in the original file */
    MqStat  FileType;     /* MQDB file format */
    MqStat  MaxAmp;       /* maximum amplitude */
    MqStat  NumOfData;    /* number of data */
    MqStat  ObsMode;      /* observation mode ( PEAKED / FLAT ) */
    MqStat  OrgFileName;  /* original data file name */
    MqStat  SampRate;     /* sampling rate */
    MqStat  Station;      /* observed station */
    MqStat  StartRecord;  /* start record number in the original file */
    MqStat  StartTime;    /* start time of the data */
    MqStat  TapeNumber;   /* original tape sequence number */
    } MqStatus;

MqFile  MqFilDesc[MQFILDESCMAX];  /* MQDB File descriptors */

/* ====================== Macro function definitions ======================= */
#define MqIsComStart(x)      (((x) >= 0x21 && (x) <= 0x2f ) || \
                              ((x) >= 0x3a && (x) <= 0x3f ) || \
                              ((x) >= 0x5b && (x) <= 0x60 ) || \
                              ((x) >= 0x7b && (x) <= 0x7e )) ?  \
                              ( MQ_TRUE ) : ( MQ_FALSE )

#endif    /* _MQDB_H */

/* =========================== END of the program ========================== */
