/* --------------- Moonquake database utilities ( header file ) --------------+
|                                                                             |
|                            "dbmqraw.h" ver 1.00                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/
#include <time.h>

/*
 *  This is a header file to be used for Moonquake data formatting program.
 *  Structure definitions relating Moonquake data raw file are stored in
 *  this file.
 */

/*
 *
 *              Explanation about Apollo Moonquake data format
 *
 *    Moonquake data obtained by Apollo mission are now summerized in files
 *  of the binary format. From below, capitalized words are only our termi-
 *  nology which are different from general use ( in computer science or
 *  seismology ).
 *    Fundamentally, file is continuum of a RECORD of 19KB ( 19,456 bytes )
 *  in length. Each RECORD is divided into a 16 bytes HEADER part and a 19440
 *  bytes DATA part. An DATA has two format type, OLD format and New format.
 *  OLD format contains LP, SP and tidal data and NEW format contains LP and
 *  tidal data ( that is, no SP data incorporated because this format is used
 *  in non-SP observation term ). An format type used in a file is recorded
 *  in file header as described below.
 *
 *                    - HEADER part configuration -
 *    HEADER part contains information about data acquisition conditions.
 *  Each information are described in 2 bytes type variable ( usually unsigned
 *  short ). Order of the variable are as follows from top of the file to the
 *  bottom.
 *     PSE:                   Passive Seismic Experiment.
 *     Apollo station number: Station number of Apollo lander ( 12 to 16 ).
 *     Original Tape Sequence number:
 *     Record number:
 *     Year:
 *     Data Format:           0 is OLD and 1 is NEW.
 *     Number of physical record from original tape:
 *     Original tape read error flags:
 *
 *                   - DATA part configuration -
 *    There are two types of formats in DATA part, OLD and NEW format, as des-
 *  cribed above. In both format, data are compartmentalized to a FIELD of
 *  72 bytes length for OLD format and 36 bytes length for NEW format. Thus,
 *  one DATA contains 270 FIELDs for OLD format and 540 FIELDs for NEW format.
 *    CELL 1 and 2 are FIELD HEADER, contains informations such as time and
 *  ALSEP tracking station ID. Seismic data are stored from CELL 3 to 18 for
 *  OLD format and 9 for NEW format ( notice that 18 for OLD and 9 for NEW is
 *  an end of the FIELD ).
 *    FIELD HEADER format is as follows:
 *
 *    bit  1:      software time flag  ( 1 bit )
 *    bit  2 - 35: time in microsecond ( 35 bits )
 *    bit 36 - 40: ALSEP tracking statin ID ( 4 bits )
 *    bit 41 - 46: bit error rate ( from original tape ) ( 6 bits )
 *    bit 47:      data rate ( 1 = 1060 bps, 0 = 530 bps ) ( 1 bit )
 *    bit 48 - 54: ( not used ) ( 7 bits )
 *    bit 55 - 64: ALSEP data word 5 ( 10 bits )
 *
 *  Seismic data are stored in 10 bits data separated by 1 bit gap. Each CELL
 *  in seismic data part contains three seismic data. Contained data are as
 *  follows:
 *
 *                 +------+---------------+---------------+
 *                 | CELL |       OLD     |      NEW      |
 *                 +------+---------------+---------------+
 *                 |   3  |               |               |
 *                 |   4  | SPZ  SPZ  SPZ | LPX  LPY  LPZ |
 *                 |   5  | LPX  SPZ  LPY | LPX  LPY  LPZ |
 *                 |   6  | SPZ  LPZ  SPZ | (1)  (2)  (3) |
 *                 |   7  | SPZ  SPZ  SPZ | LPX  LPY  LPZ |
 *                 |   8  | SPZ  SPZ  LPX | (4)  LPX  LPY |
 *                 |   9  | SPZ  LPY  SPZ | LPZ           |
 *                 |  10  | LPZ  SPZ  SPZ +---------------+
 *                 |  11  | (1)  SPZ  (2) |  (1) house keeping
 *                 |  12  | SPZ  (3)  LPX |  (2) even FIELD: TIDAL X
 *                 |  13  | SPZ  LPX  SPZ |       odd FIELD: TIDAL Z
 *                 |  14  | LPY  SPZ  LPZ |  (3) even FIELD: TIDAL Y
 *                 |  15  | (4)  SPZ  SPZ |       odd FIELD: temp.*
 *                 |  16  | SPZ  SPZ  LPX |  (4) command verification
 *                 |  17  | SPZ  LPY  SPZ |  (5) ALSEP number
 *                 |  18  | LPZ  SPZ  SPZ |
 *                 +------+---------------+
 *                  *: instrumental temperature
 *
 *  Thus in each FIELD, there are four sets of LP data and 29 SP data. TIDAL
 *  data completes by two FIELDs.
 *
 *  For more detail, see:
 *   Jun-ya Terazono, 1993, Deep Moonquake Analyses for Lunar Missions, Master
 *  thesis of University of Tokyo.
 *    Fig. 1.3 has bugs about expression for OLD data format cell 8 ( 3rd com-
 *  ponent should be LPX ) and OLD/NEW data format cell 1 and 2 ( time parts
 *  should be longer ). Table above is correct.
 *    Structure definitions below are also helpful for comprehension of data
 *  formats.
 */

/* =========================== Macro declarations ========================== */

    /* constants about file structures */
#define RECORDSIZE    19456    /* length of the one data field */
#define RECORDMAX       999    /* tolerable maximum record number */
#define HEADERSIZE       16    /* length of the header */
#define FIELD_NUM_OLD   270    /* number of FIELDs ( OLD format ) */
#define FIELD_NUM_NEW   540    /* number of FIELDs ( NEW format ) */
#define FIELD_LEN_OLD    72    /* length of FIELD ( OLD format ) */
#define FIELD_LEN_NEW    36    /* length of FIELD ( NEW format ) */
#define CELL_NUM_OLD     18    /* number of CELLs in one FIELD ( OLD ) */
#define CELL_NUM_NEW      9    /* number of CELLs in one FIELD ( NEW ) */
#define OLDRECTIME      163.5  /* average record range ( OLD ) */
#define NEWRECTIME      326.5  /* average record range ( NEW ) */
#define GAPLIMIT          1.0  /* tolerable limit of record gaps */
#define RANGELIMIT     3600.0  /* tolerable limit of record time range */
#define LP_DATA_NUM      4     /* number of LP data in one FIELD */
#define SP_DATA_NUM     29     /* number of SP data in one FIELD */
#define TIDAL_DATA_NUM   3     /* number of TIDAL data */

/* ========================= Structure declarations ======================== */

   /*  Definition of the data structure in the CELL */
typedef struct {
    unsigned int      data1   :10;
    unsigned int      gap1    : 1;
    unsigned int      data2   :10;
    unsigned int      gap2    : 1;
    unsigned int      data3   :10;
    }  DATACELL;

typedef union  {
    unsigned char     cell[4];
    DATACELL         datacell;
    }  CELL;

    /* Definition of the data structure in FIELD HEADER */
    /* length is 64 bit == 8 bytes */
typedef struct {
    unsigned int       timeflag  : 1;   /* software time flag */
#ifdef NOLONGLONG
    unsigned long         time1  :31;   /* upper part of time in microsecond */
    unsigned long         time2  : 4;   /* lower part of time in microsecond */
#else
    unsigned long long     time  :35;   /* time in microsecond */
#endif
    unsigned int          alsep  : 4;   /* ALSEP tracking station ID */
    unsigned int         errate  : 6;   /* bit error rate */
    unsigned int       datarate  : 1;   /* data rate */
    unsigned int         unused  : 7;   /* this space is not used */
    unsigned int       dataword  :10;   /* ALSEP data word */
    }  FIELDHEADER;

    /* Definition of the data structures in the FIELD */
union field_old   {    /* OLD format */
    FIELDHEADER     fheader;
    CELL            cell[CELL_NUM_OLD];
    };

union field_new    {    /* NEW format */
    FIELDHEADER    fheader;
    CELL           cell[CELL_NUM_NEW];
    };

union field_old_union    {    /* OLD format */
    char              field_old[FIELD_LEN_OLD];
    union field_old  field;
    };

union field_new_union    {    /* NEW format */
    char              field_new[FIELD_LEN_NEW];
    union field_new  field;
    };

typedef union field_old_union      FIELD_OLD;
typedef union field_new_union      FIELD_NEW;

   /* Definition of the HEADER data structure */

struct header    {
    unsigned short    pse;        /* PSE number */
    unsigned short    station;    /* Apollo seismic station number */
    unsigned short    seqno;      /* Original tape Sequence number */
    unsigned short    recno;      /* Record number */
    unsigned short    year;       /* year */
    unsigned short    dataform;   /* data format ( 0: old, 1: new ) */
    unsigned short    physrec;    /* Number of physical record */
    unsigned short    errorflag;  /* Original tape read error flag */
    };

typedef struct header   HEADER;

    /* Definition of RECORD data structure */
struct record_old   {    /* OLD format */
    HEADER    header;
    FIELD_OLD field[FIELD_NUM_OLD];
    };

struct record_new   {    /* NEW format */
    HEADER    header;
    FIELD_NEW field[FIELD_NUM_NEW];
    };

typedef struct record_old    RECORD_OLD;
typedef struct record_new    RECORD_NEW;

union record    {
    char    record[RECORDSIZE];
    HEADER  header;
    RECORD_OLD  record_old;
    RECORD_NEW  record_new;
    };

typedef union record    RECORD;

/* =========================== END of the program ========================== */

