/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                              "mqsac.h" ver 1.00                             |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1994-1995 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This file contains essential definitions of macro and static variables to 
 *  work the program "mqsac" properly.
 */

/* ============================= Include files ============================= */
#include <mqdb.h>

/* =========================== Macro definitions =========================== */

    /* boolean macros */
#define TRUE     1
#define FALSE    0

    /* option */
#define OPTLEN  16    /* length of option strings */

    /* ITYPE (file type) header specification */
#define ITIME    0    /* time series file */
#define IRLIM    1    /* spectral file: real and imaginary */
#define IAMPH    2    /* spectral file: amplitude and phase */
#define IXY      3    /* general x-y file */
#define IXYZ     4    /* general xyz file */

    /* IDEP (variable type) header specification */
#define IUNKN    0    /* unknown */
#define IDISP    1    /* displacement in nm */
#define IVEL     2    /* velocity in nm/sec */
#define IVOLTS   3    /* velocity in volts */
#define IACC     4    /* acceleration in nm/sec/sec */

    /* IZTYPE (reference time equivalence) header specification */
#define IB       1    /* Begin time */
#define IDAY     2    /* midnight of reference GMT day */
#define IO       3    /* event origin time */
#define IA       4    /* first arrival time */
#define IT0      5    /* user-defined time pick 0 */
#define IT1      6    /* user-defined time pick 1 */
#define IT2      7    /* user-defined time pick 2 */
#define IT3      8    /* user-defined time pick 3 */
#define IT4      9    /* user-defined time pick 4 */
#define IT5     10    /* user-defined time pick 5 */
#define IT6     11    /* user-defined time pick 6 */
#define IT7     12    /* user-defined time pick 7 */
#define IT8     13    /* user-defined time pick 8 */
#define IT9     14    /* user-defined time pick 9 */

    /* IEVTYP (type of event) header specification */
#define INUCL    1    /* nuclear event */
#define IPREN    2    /* nuclear pre-shot event */
#define IPOSTN   3    /* nuclear post-shot event */
#define IQUAKE   4    /* earthquake */
#define IPREQ    5    /* foreshock */
#define IPOSTQ   6    /* postshock */
#define ICHEM    7    /* chemical explosion */
#define IOTHER   8    /* other */

    /* error macros */
#define EMSNOERROR    0  /* no error */
#define EMSNOARGS     1  /* no arguments */
#define EMSMANYARGS   2  /* too many arguments */
#define EMSREADFAIL   3  /* MQDB file reading failed */
#define EMSMEMALLOC   4  /* memory allocation failed */
#define EMSILDATATYPE 5  /* illegal data type */
#define EMSILOPT      6  /* illegal option */
#define EMSDELTA      7  /* sampling rate must be a same */
#define EMSILMQTIME   8  /* illegal MqTime value */
#define EMSERROR      9  /* some error */
#define EXITLEVEL     9  /* exits error numbered less than this */

    /* optional function */
#define VERBOSE       1  /* verbose output */
#define HELP          2  /* help output */

/* ============ Structure and Enumeration Variable Declarations  =========== */
typedef enum { MS_WARNING, MS_FATAL, MS_HELP } Level;   /* error level */

typedef struct {   /* argument specifications */
    char           opt[16];   /* argument */
    unsigned short func;      /* function number */
    MqYesNo        optargs;   /* if YES, this option has parameter */
    } OPTIDX;

typedef struct {   /* error handling structure */
    char           errmsg[80];  /* error message */
    unsigned short errno;       /* error number */
    Level          level;       /* level of this error */
    } ERROR;

/* ========================= Variable declarations ========================= */
static char progsign[] = "mqsac-1.00";   /* program signature */

static OPTIDX optidx[] =           /* argument specifications */
    { "v",        VERBOSE,  MQ_NO,
      "help",     HELP,     MQ_NO };

static ERROR  error[] =    /* error message and attributions */
   { "Program exited with no error", EMSNOERROR,    MS_WARNING,
     "No arguments",                 EMSNOARGS,     MS_HELP,
     "Too many arguments",           EMSMANYARGS,   MS_FATAL,
     "MQDB file reading failed",     EMSREADFAIL,   MS_FATAL,
     "Memory allocation error",      EMSMEMALLOC,   MS_FATAL,
     "Illegal data type",            EMSILDATATYPE, MS_WARNING,
     "Illegal option",               EMSILOPT,      MS_FATAL,
     "Sampling interval must be same", EMSDELTA,    MS_WARNING,
     "Illegal MqTime value included",  EMSILMQTIME, MS_FATAL,
     "An error occured" ,            EMSERROR,      MS_FATAL
	 };

    /* file name extension for SAC output file */
static char SACext[] = ".sac";

    /*
     *  name of the header fields used in SAC-format file 
     *    WARNING: every string must be 6 in length.     
     *    If length of strings are less than this, space must be padded
     *    as described below.
     *  Please check a manual of SAC for detailed meanings of each fields.
     *  In the following comments, each one capital surrounded in brace ()
     *  represents data type:
     *     (N)  Integer ( 32-bit length. "long" ).
     *     (F)  Floating variable, single precision ( "float" ).
     *     (I)  Enumerated variable ( NOT equivalent to "enum" used in C ).
     *     (L)  Logical variable ( LOGICAL in Fortran ).
     *     (K)  Alphanumeric variable ( "char *" ).
     *     (A)  Auxiliary.
     *  Headers that are not currently used are omitted from the list.
     */

static char field_npts[]   = "NPTS  ";  /* Number of points per data (N) */
static char field_b[]      = "B     ";  /* Beginning value (F) */
static char field_e[]      = "E     ";  /* Ending value (F) */
static char field_iftype[] = "IFTYPE";  /* Type of file (I) */
static char field_leven[]  = "LEVEN ";  /* TRUE if data is evenly-spaced (L) */
static char field_delta[]  = "DELTA ";  /* Increment value (F) */
static char field_odelta[] = "ODELTA";  /* Observed increment (F) */
static char field_idep[]   = "IDEP  ";  /* Type of dependent variable (I) */
static char field_depmin[] = "DEPMIN";  /* Min value (F) */
static char field_depmax[] = "DEPMAX";  /* Max value (F) */
static char field_depmen[] = "DEPMEN";  /* Mean value (F) */
static char field_nzyear[] = "NZYEAR";  /* GMT year (N) */
static char field_nzjday[] = "NZJDAY";  /* GMT julian day (N) */
static char field_nzhour[] = "NZHOUR";  /* GMT hour (N) */
static char field_nzmin[]  = "NZMIN ";  /* GMT minute (N) */
static char field_nzsec[]  = "NZSEC ";  /* GMT second (N) */
static char field_nzmsec[] = "NZMSEC";  /* GMT millisecond (N) */
static char field_nzdttm[] = "NZDTTM";  /* GMT date-time array (N) */
static char field_kzdate[] = "KZDATE";  /* Alphanumeric form of date (A) */
static char field_kztime[] = "KZTIME";  /* Alphanumeric form of time (A) */
static char field_o[]      = "O     ";  /* Origin time (F) */
static char field_ko[]     = "KO    ";  /* Origin time identification (A) */
static char field_a[]      = "A     ";  /* First arrival time (F) */
static char field_ka[]     = "KA    ";  /* First arrival time ident. (A) */
static char field_f[]      = "F     ";  /* Fini or end of event time (F) */
static char field_kf[]     = "KF    ";  /* Fini identification (A) */
static char field_iztype[] = "IZTYPE";  /* Reference time equivalence (I) */
static char field_kinst[]  = "KINST ";  /* Recording instrument name (K) */
static char field_knetwk[] = "KNETWK";  /* Name of seismic network (K) */
static char field_kstnm[]  = "KSTNM ";  /* Station name (K) */
static char field_stla[]   = "STLA  ";  /* Station latitude (N positive) (F) */
static char field_stlo[]   = "STLO  ";  /* Station longitude (E positive)(F) */
static char field_cmpaz[]  = "CMPAZ ";  /* Azimuth (clockwise from N) (F) */
static char field_cmpinc[] = "CMPINC";  /* Incident angle (F) */
static char field_kcmpnm[] = "KCMPNM";  /* Component name (A) */
static char field_kstcmp[] = "KSTCMP";  /* Station component (K) */
static char field_lpspol[] = "LPSPOL";  /* TRUE if positive polarity (L) */
static char field_kevnm[]  = "KEVNM ";  /* Event name (K) */
static char field_evla[]   = "EVLA  ";  /* Event latitude (N positive) (F) */
static char field_evlo[]   = "EVLO  ";  /* Event longitude (E positive) (F) */
static char field_ievtyp[] = "IEVTYP";  /* Type of event (I) */
static char field_khole[]  = "KHOLE ";  /* Hole identification (K) */
static char field_dist[]   = "DIST  ";  /* STN-EVENT distance (F) */
static char field_az[]     = "AZ    ";  /* EVENT-STN azimuth (F) */
static char field_baz[]    = "BAZ   ";  /* STN-EVENT azimuth (F) */
static char field_gcarc[]  = "GCARC ";  /* STN-EVENT GCA length (F) */
static char field_lcalda[] = "LCALDA";  /* TRUE if DIST etc. to be cal. (L) */
static char field_kdatrd[] = "KDATRD";  /* Date data was processed (K) */
static char field_lovrok[] = "LOVROK";  /* TRUE if file can overwrite (L) */
static char field_nvhdr[]  = "NVHDR ";  /* Header version number (N) */

    /* Enumeration variable */
static char *iftype[] = {        /* IFTYPE enumeration ( type of file ) */
    "ITIME ",    /* time series file */ 
    "IRLIM ",    /* spectral file: real and imaginary */
    "IAMPH ",    /* spectral file: amplitude and phase */
    "IXY   ",    /* general x-y data */
    "IXYZ  " };  /* general xyz data */

static char *idep[]  = {         /* IDEP enumeration (type of variable) */
    "IUNKN ",    /* unknown */
    "IDISP ",    /* displacement in nm */
    "IVEL  ",    /* velocity in nm/sec */
    "IVOLTS",    /* velocity in volts */
    "IACC  " };  /* acceleration in nm/sec/sec */

static char *iztype[] = {        /* IZTYPE enumeration (reference time ) */
    "IUNKN ",    /* unknown */
    "IB    ",    /* begin time */
    "IDAY  ",    /* midnight of reference GMT day */
    "IO    ",    /* event origin time */
    "IA    ",    /* first arrival time */
    "IT0   ",    /* user-defined time pick 0 */
    "IT1   ",    /* user-defined time pick 1 */
    "IT2   ",    /* user-defined time pick 2 */
    "IT3   ",    /* user-defined time pick 3 */
    "IT4   ",    /* user-defined time pick 4 */
    "IT5   ",    /* user-defined time pick 5 */
    "IT6   ",    /* user-defined time pick 6 */
    "IT7   ",    /* user-defined time pick 7 */
    "IT8   ",    /* user-defined time pick 8 */
    "IT9   " };  /* user-defined time pick 9 */

static char *ievtyp[] = {       /* IEVTYP enumeration (event type) */
    "IUNKN ",    /* unknown */
    "INUCL ",    /* nuclear event */
    "IPREN ",    /* nuclear pre-shot event */
    "IPOSTN",    /* nuclear post-shock event */
    "IQUAKE",    /* earthquake */
    "IPREQ ",    /* foreshock */
    "IPOSTQ",    /* aftershock */
    "ICHEM ",    /* chemical explosion */
    "IOTHER" };  /* other */

    /* parameter storage that used in this program */
struct {
    char     progname[16];    /* name of this program */
    char     infile[255];     /* input file name */
    char     outfile[255];    /* output file name */
    MqYesNo  help;            /* help message output */
    MqYesNo  verbose;         /* verbose output */
    } params;     	      /* program parameters */

static char helpmsg[] =
    "Usage: mqsac [ -help ] [ -v ] MQDBfile SACfile\n"
    "       mqsac [ -help ] [ -v ] MQDBfilebase\n";
    
/* ========================= Function declarations ========================= */
char * ExtractExt( char * );
void   DisplayError( char *, unsigned short );
void   InitParams( void );
void   MakeOutFileNameBase( char *, char * );
void   OutputSACfileLP( MqHeader *, double * );
void   OutputSACfileSP( MqHeader *, double * );
void   OutputSACfileTDL( MqHeader *, double * );
long   SetDELTA( MqAmp );
void   SetOptions( int, char ** );
void   SetParams( unsigned short, char * );
long   SetTime( MqTime );

/* =========================== END of the program ========================== */

