/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                               "wm2mq" ver 1.00                              |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|        Copyright (c) 1993-1994 Jun-ya Terazono, all rights reserved.        |
+----------------------------------------------------------------------------*/

/*
 *  This program reads WaveMaster 1.x file ( xxx.wmas and xxx.info: xxx must 
 *  be the same ) and outputs MQDB format file ( filename will be xxx.mqf ).
 */

/* ============================= Include files ============================= */
#include "wm2mq.h"

/* ========================== Program statements  ========================== */
main( int argc, char **argv )
    {
    char   wmasfile[255],    /* WaveMaster data file */
           infofile[255],    /* WaveMaster information file */
           mqffile[255];     /* MQDB format file */
    double *data,            /* actual data */
            samprate;        /* sampling rate */
    unsigned long   linenum,   /* number of lines in *.wmas file */
                    datanum;   /* number of whole data */
    unsigned short  channels;  /* number of channels */
    MqAccess   access;       /* header access control structure */
    MqHeader   header;       /* header contents */

    MqInit( progsign );

       /* argument number check ( only 1 allowed ) */
    if( argc < 2 )       OutputError( EWMQNOARGS );
    else if( argc > 2 )  OutputError( EWMQMANYARGS );
 
    MakeFileNames( wmasfile, infofile, mqffile, argv[1] );

        /* Notice that first line of .wmas file is a time interval, that is, 
           number of actual data are channels * ( linenum - 1 ). */
    linenum = CountLineNumber( wmasfile );
    channels = CountDataSeparator( wmasfile );
    datanum = ( linenum - 1 ) * (unsigned long)channels;

    if( NULL == ( data = calloc( datanum, sizeof( double ))))
       OutputError( EWMQMEMALLOC );

    samprate = ReadWmasData( wmasfile, linenum, data );

    header = SetMqHeader( datanum, channels, wmasfile, samprate, data );
    access = SetMqAccess();

    OutputMqFile( mqffile, &access, &header, data );
    OutputParams( &header );
    }

/* ===== Error message output ===== */
unsigned short OutputError( unsigned short error )
    {
    fprintf( stderr, "wm2mq: " );
    fprintf( stderr, "%s\n", errormsg[error] );
    if( error == EWMQNOARGS || error == EWMQMANYARGS )
	fprintf( stderr, "usage: wm2mq [ file ]\n" );

    exit( error );
    }

/* ===== Make file names for WaveMaster and MQDB format ===== */
void MakeFileNames( char *wmasfile, char *infofile, char *mqffile,
		    char *basename )

/*
 *  This function creates file names necessary for reading and outputting.
 *  arguments are:
 *      wmasfile:  file name for WaveMaster data file.
 *      infofile:  file name for WaveMaster info file.
 *      mqffile:   file name for MQDB format file.
 *      basename:  file name base assigned from command line.
 */

    {
    unsigned long  arglen;     /* length of original argument */

    arglen = strlen( basename );
    if( arglen > 250 )
	OutputError( EWMQLONGFILE );

    if( NULL == strcpy( wmasfile, basename ))  OutputError( EWMQNULLPTR );
    if( NULL == strcpy( infofile, basename ))  OutputError( EWMQNULLPTR );
    if( NULL == strcpy( mqffile,  basename ))  OutputError( EWMQNULLPTR );

    if( NULL == strcat( wmasfile, wmasext ))  OutputError( EWMQNULLPTR );
    if( NULL == strcat( infofile, infoext ))  OutputError( EWMQNULLPTR );
    if( NULL == strcat( mqffile, mqfext ))    OutputError( EWMQNULLPTR );
    }

/* ===== Count Number of Lines in *.wmas file ===== */
unsigned long CountLineNumber( char *wmasfile )

/* 
 * This function counts number of lines in *.wmas file. If line number is zero
 * or one, error handling routine will be called to terminate the whole pro-
 * gram, otherwise counted line number will be returned.
 */

    {
    unsigned long    linenum = 0;    /* count(ed) line number */
    FILE    *fwmas;                  /* file descriptor */

    if( NULL == wmasfile ) OutputError( EWMQNULLFILE );

   if( NULL == ( fwmas = fopen( wmasfile, "r" )))   {
       perror( wmasfile );
       OutputError( EWMQOPENWMAS );
       }

    while( !feof( fwmas ))   {
	if( 0x0a == getc( fwmas )) linenum++;
	}

    fclose( fwmas );

    if( linenum == 0 )       OutputError( EWMQNOLINE );
    else if ( linenum == 1 ) OutputError( EWMQONELINE );
    else                     return linenum;
    }

/* ===== Count Number of Lines in *.wmas file ===== */
unsigned short CountDataSeparator( char *wmasfile )

/*
 *  This function reads a top line of WaveMaster data file and counts number
 *  of data separator ','. Notice that this function only specifies first 
 *  line only, which assumes that all lines in the input file has same sepa-
 *  rator formats.
 */

    {
    unsigned short     channels = 1;    /* count(ed) channels */
    FILE    *fwmas;                     /* file descriptor */
    unsigned long      i = 0,           /* loop counter */
                       toplinelen = 0;
    char     topline[LINEMAX];      /* first line of the input file */

    if( NULL == wmasfile ) OutputError( EWMQNULLFILE );

    if( NULL == ( fwmas = fopen( wmasfile, "r" )))   {
       perror( wmasfile );
       OutputError( EWMQOPENWMAS );
       }

    if( NULL == fgets( topline, LINEMAX, fwmas ))
	OutputError( EWMQGETLINE );

    toplinelen = strlen( topline );
    if( toplinelen == 0 )  OutputError( EWMQNOLINECONT );

    for( i = 0; i < toplinelen; i++ )
	if( ',' == topline[i] ) channels++;

    return channels;
    }

/* ===== Read WaveMaster data file ===== */
double ReadWmasData( char *wmasfile, unsigned long linenum, double *data )

/*
 *  This function reads WaveMaster data file and stores sampling rate and
 *  actual data value. Sampling rate will be returned and actual data
 *  will be set to the array "data". Notice that this function does not 
 *  check equality of values of sampling rates in each channel.
 */

    {
    double  samprate,           /* sampling rate */
            dummy1, dummy2;     /* dummy variable for samprate reading */
    FILE   *fwmas;              /* file descriptor */
    unsigned long   i;          /* loop counter */
    
    if( NULL == ( fwmas = fopen( wmasfile, "r" )))   {
       perror( wmasfile );
       OutputError( EWMQOPENWMAS );
       }

    fscanf( fwmas, "%lf, %lf, %lf", &samprate, &dummy1, &dummy2 );

    linenum--;

    for( i = 0; i < ( linenum - 1 ); i++ )    {
	fscanf( fwmas, "%lf, %lf, %lf", 
	       data+i, data+linenum+i, data+linenum*2+i );
	}

    return samprate;
    }

/* ===== Set MQDB header access control variable ===== */
MqAccess SetMqAccess( void )

/*
 *  This function sets MqAccess structure ( MQDB header access control struc-
 *  ture ) using MQLIB function MqSetAccessNo. Specific headers will be set
 *  to MQ_YES.
 */

    {
    MqAccess   access;    /* MQDB header access control structure */

    MqSetAccessNo( &access );       /* Initialize all to MQ_NO */

    access.AveAmp = MQ_YES;
    access.Channels = MQ_YES;
    access.DataCreDate = MQ_YES;
    access.MaxAmp = MQ_YES;
    access.NumOfData = MQ_YES;
    access.OrgFileName = MQ_YES;
    access.SampRate = MQ_YES;

    return access;
    }

/* ===== Set MqHeader variable ===== */
MqHeader SetMqHeader( unsigned long datanum, unsigned short channels,
		     char *orgfile, double samprate, double *data )
    {
    MqHeader   header;
    unsigned long    cwdlen,   /* length of the string of cwd */
                     i,        /* loop counter */
                     linenum;  /* number of data in one channel */
    char       cwd[LINEMAX];   /* current working directory */

    header.NumOfData = datanum;
    header.Channels =  channels;
    header.SampRate.channels = 1;
    header.AveAmp.channels = header.MaxAmp.channels = channels;
    header.SampRate.data[0]  = samprate;
    linenum = datanum / channels;

    for( i = 0; i < channels; i++ )   {
	header.AveAmp.data[i] = MqCalcAveAmp( data, linenum*i, linenum );
	header.MaxAmp.data[i] = MqCalcMaxAmp( data, linenum*i, linenum );
	}

         /* get current directory name to add original file name */
    if( NULL == getcwd( cwd, LINEMAX ))   {
	perror( NULL );
	OutputError( EWMQNULLPTR );
	}

    strcpy( header.OrgFile, cwd );
    strcat( header.OrgFile, "/" );
    strcat( header.OrgFile, orgfile );

    /* copy data and calculate average/maximum amplitude here */
    /* execute time function to get current time */

    return header;
    }

/* ===== Output MQDB Format file ===== */
void OutputMqFile( char *outfile, MqAccess *access,
		  MqHeader *header, double *data )
    {
    MqCreate( outfile, access, header, data );

    if( MqErr.errno == EMQNOERROR )
	fprintf( stderr, "Data output completed.\n" );
    else
	fprintf( stderr, "Data output failed.\n" );

    }

/* ===== Output MQDB Parameters ===== */
void OutputParams( MqHeader *header )
    {
    fprintf( stderr, "=============================="
	             " Output Parameters "
	             "=============================\n" );
    fprintf( stderr, "Number of channels: %d\n", header->Channels );
    fprintf( stderr, "Number of Data: %ld\n", header->NumOfData );
    fprintf( stderr, "Original file name: %s\n", header->OrgFile );
    fprintf( stderr, "Sampling rate: %lf\n", header->SampRate.data[0] );
    fprintf( stderr, "========================================"
                     "======================================\n" );
    }

/* =========================== END of the program ========================== */

