/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                               "wpr.c" ver 2.20                              |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|      Copyright (c) 1993-1994 by Jun-ya Terazono, all rights reserved.       |
+----------------------------------------------------------------------------*/

/* 
 *   This is a utility program for Moonquake database to output wave to the 
 *   printer (currently LaserShot LIPS II mode is supported). This file 
 *   accepts file formats of WaveMaster readings.
 *   This file contains main routine and general functions of wpr.
 *   Please compile with ANSI C supported compilers such as GNU cc and SUN C.
 */

/* ========================= History of modifications ======================== 
1.00  (93.05.18)  MS-DOS version written by Hitoshi Mizutani and J.T.
1.00  (93.06.18)  First UNIX version implemented by J.T.

2.00  (94.05.31)  Became MQDB-compatible.
2.10  (94.07.16)  New option: -page ( assigned page output ).
                  Option systems rearranged.
2.20  (94.07.21)  New option: -help ( display help message ).
============================================================================ */

/* ============================= Include files ============================= */
#include "wpr.h"
#include "wprerror.h"

/* =========================== Program statements ========================== */
main( int argc, char **argv )
    {
    PARAMS   params;    /* command line parameters */
    DRAW     draw;      /* drawing parameters */
    double  *data,      /* MQDB original data */
             *newdata;  /* data after rearrangement */
    MqHeader header;    /* MQDB header contents */
    MqAccess access;    /* MQDB header access control structure */

    MqInit( "wpr" );
    MqSetWarnLevel( EMQ_FATAL );

    GetParams( argc, argv, &params );

    MqSetAccessYes( &access );     /* reads all header */

    if( params.opts.infile == INPUT_STDIN ) {
	if( NULL == ( data = MqRead( NULL, &access, &header ))) {
	    MqPerror( NULL );
	    wprerror( EWPRMQREAD );
	    }
	}
    else {
	if( NULL == ( data = MqRead( params.file, &access, &header ))) {
	    MqPerror( NULL );
	    wprerror( EWPRMQREAD );
	    }
	}

    draw = SetDraw( &params, &header );

    if( params.opts.verbose >= VERBOSE ) OutputDrawParams( &draw );

    if( NULL == ( newdata = RearrangeData( &draw, &params, data )))
       wprerror( EWPRNOMEMORY );

    switch( params.opts.page )    {
    case TRUE:
	draw.currpage = params.pagenum;
	printout( &draw, &params, &header, newdata );
	break;
    case FALSE:
    for( draw.currpage = 1; draw.currpage <= draw.pages;
	draw.currpage++ )
	printout( &draw, &params, &header, newdata );
	break;
	}

    L2DrawEnd( &params );
    }	

/* ===== Get options and returns current status ===== */
void GetParams( int argc, char **argv, PARAMS *params )

 /*
  *  This is a function for evaluating command line options and arguments
  *    argc: number of command line arguments
  *    argv: values of arguments
  *    params: parameter structure
  */

    {
    int    i, j;      /* loop counter */
    short  optattr,
           argnum = 0;
    char   optchars[255];

    SetDefaultParams( params );

    for( i = 1; i < argc; i++ )   {
	if( *argv[i] == '-' )    {
	    strcpy( optchars, *(argv+i)+1 );
	    for( j = 0; j < WPROPTS; j++ )    {
		if( 0 == ( strcmp( optchars, optlist[j] )))    {
		    optattr = optfunc[j];
		    
		    switch( optattr )    {
		    case HELP:
			DisplayUsage( EWPRNOERROR );
			exit( EWPRNOERROR );
		    case VERBOSE:            /* verbose */
			params->opts.verbose = VERBOSE;
			argnum++;
			break;
		    case VERYVERBOSE:        /* very verbose */
			params->opts.verbose = VERYVERBOSE;
			argnum++;
			break;
		    case PAGE:
			params->opts.page = TRUE;
			params->pagenum = atol( argv[i+1] );
			argnum++; argnum++;
			j++;
			break;
		    case AMPSCALE:
			params->opts.ampscale = TRUE;
			params->ampscale = atof( argv[i+1] );
			argnum++; argnum++;
			j++;
			break;
		    case TIMESCALE:
			params->opts.timescale = TRUE;
			params->timescale = atof( argv[i+1] );
			argnum++; argnum++;
			j++;
			break;
			}
		    }
		}
	    }
	}

    if( argc - 1 == argnum ) {    /* standard input is assumed */
	if( params->opts.verbose >= VERBOSE )
	    fprintf( stderr, "WARNING: we assume standard input.\n" );
	params->opts.infile = INPUT_STDIN;
	}
    else {
	if( params->opts.verbose == VERYVERBOSE )
	    fprintf( stderr, "Input file is %s\n", argv[argc-1] );
	strcpy( params->file, argv[argc-1] );
	params->opts.infile = INPUT_FILE;
	}

    CheckParams( params );
    }

/* ===== Error handling routine ===== */
void wprerror( int errornum )
    {
    fprintf( stderr, "wpr: " );
    
    switch( errornum )    {
    case EWPRNOERROR:
	break;
    case EWPRNOARGS:
	fprintf( stderr, "No file name.\n" );
	DisplayUsage( errornum );
	break;
    case EWPRFILOPEN:
	fprintf( stderr, "Failed to open input file.\n" );
	break;
    case EWPRILOPT:
	fprintf( stderr, "Illegal option.\n" );
	DisplayUsage( errornum );
	break;
    case EWPRNOMEMORY:
	fprintf( stderr, "Memory allocation failed.\n" );
	break;
    case EWPRSAMPRATE:
	fprintf( stderr, "Sampling rates differ in channels.\n" );
	break;
    case EWPRZEROCH:
	fprintf( stderr, "Number of channels are zero.\n" );
	break;
    case EWPRMQREAD:
	fprintf( stderr, "MqRead() failed.\n" );
	break;
    default:
	fprintf( stderr, "Error occured ( unknown reasons ).\n" );
	break;
	}

    exit( errornum );
    }

/* ===== output of DisplayUsage ===== */
void DisplayUsage( int errornum )
    {
    if( errornum == EWPRNOERROR )    {
	fprintf( stderr, "wpr is a program to output MQDB-format wave data to printer.\n " );
	fprintf( stderr, "Currently supported output printer is LIPS II+.\n" );
	}

    fprintf( stderr, "  usage: wpr [ options ] filename\n" );
    fprintf( stderr, "    -help:        display this message.\n" );
    fprintf( stderr, "    -page N:      output page. N is a page number.\n" );
    fprintf( stderr, "    -v, -verbose: verbose output\n" );
    fprintf( stderr, "    -V, -Verbose: very verbose output\n" );
    fprintf( stderr, "    -a, -ampscale: change amplitude scale\n" );
    fprintf( stderr, "    -t, -timescale: change time scale\n" );
    }

/* ===== set DRAW parameters ===== */
DRAW SetDraw( PARAMS *params, MqHeader *header )

/*
 *  This function sets parameters for wave drawings such as page number and
 *  number of frames.
 */

    {
    long   i;       /* loop counter */
    DRAW   draw;
    
    draw.datanum = header->NumOfData / header->Channels;

/*
 *  Points per 1 page are set. If this value is assigned by command line 
 *  option, that value is used. Otherwise, default value ( 2520 ) is used.
 *  Current support are default value only.
 */

    draw.points.ord = DEFAULT_POINTS;

       /* time interval value are set. */
    if( header->SampRate.channels == 1 )
	draw.timeint = header->SampRate.data[0];
    else
	wprerror( EWPRSAMPRATE );

    draw.channels = header->Channels;       /* channel number */

    SetDrawParams( &draw );       /* drawing parameters are set */

        /* plotting offset value are set */
    SetOffset( &draw );
    for( i = 0; i < header->AveAmp.channels; i++ )
	draw.aveamp[i] = header->AveAmp.data[i];

        /* output date are set as a string */
    GetOutputDate( &draw );
    if( params->opts.verbose >= VERBOSE )
	fprintf( stderr, "Data printing started on %s\n", draw.outtime );
    
/*
 *  Amplitude scale and time scale are set. If option is not assigned, 
 *  default values are used instead.
 */

    ( params->opts.ampscale == TRUE ) ?
	( draw.ampscale = params->ampscale ) :
	    ( draw.ampscale = DEFAULT_AMPSCALE );
    ( params->opts.timescale == TRUE ) ?
	( draw.timescale = params->timescale ) :
	    ( draw.timescale = DEFAULT_TIMESCALE );

    return draw;
    }

/* ===== calculate number of frames and points of the last frame ===== */
FRPTS SetFrameLastPts( unsigned long points )

/* 
 *  This function calculates number of frames and points of the last frame
 *  and returns structure FRPTS.
 */
    {
    FRPTS   frpts;

    frpts.frames = 1;    /* number of frames */
    while( points > FRAME_LENGTH )    {
	points = points - FRAME_LENGTH + 1;
	frpts.frames++;
	}
    frpts.lastpts = points;

    return frpts;
    }

/* ===== get output time and format to strings ===== */
void GetOutputDate( DRAW *draw )
    {
    char       *timestr, newtime[24];
    time_t      drawtime;
    int         timelen;     /* length of timelen */

    time( &drawtime );
    timestr = ctime( &drawtime );

    timelen = strlen( timestr );
    strncpy( newtime, timestr, timelen - 1 );
    newtime[timelen-1] = '\0';

    strcpy( draw->outtime, newtime );
    }
    
/* ===== parameter checkings ===== */
void CheckParams( PARAMS *params )
    {
    if( params->opts.ampscale == FALSE )
	params->ampscale = DEFAULT_AMPSCALE;

    if( params->opts.timescale == FALSE )
	params->timescale = DEFAULT_TIMESCALE;

    if( params->opts.infile == INPUT_ERROR )
	wprerror( EWPRNOINFILE );
    }

/* ===== output parameter if verbose mode is on ===== */
void OutputDrawParams( DRAW *draw )
    {
    fprintf( stderr, "===================================================\n" );
    fprintf( stderr, "Time interval is %f\n", draw->timeint );
    fprintf( stderr, "Points per 1 page ( ord )  : %ld\n", draw->points.ord );
    fprintf( stderr, "Points per 1 page ( last ) : %ld\n", draw->points.last );
    fprintf( stderr, "Number of frames  ( ord )  : %ld\n", draw->frames.ord );
    fprintf( stderr, "Number of frames  ( last ) : %ld\n", draw->frames.last );
    fprintf( stderr, "Points in the last frame ( ord ): %ld\n",
	draw->lastpts.ord );
    fprintf( stderr, "Points in the last frame ( last ): %ld\n",
	draw->lastpts.last );
    fprintf( stderr, "Channels in data are %ld\n", draw->channels );
    fprintf( stderr, "Number of pages are  %ld\n", draw->pages );
    fprintf( stderr, "Data numbers are     %ld\n", draw->datanum );
    fprintf( stderr, "Amplitude scale is   %lf\n", draw->ampscale );
    fprintf( stderr, "Time scale is        %lf\n", draw->timescale );
    fprintf( stderr, "Window height is     %lf\n", draw->winheight );
    fprintf( stderr, "==================================================\n" );
    }

/* ===== settings of default parameters ===== */
void SetDefaultParams( PARAMS *params )
    {
    params->opts.verbose   = SILENT;
    params->opts.infile    = INPUT_ERROR;
    params->opts.page      = FALSE;
    params->opts.ampscale  = FALSE;
    params->opts.timescale = FALSE;
    }

/* ===== set offset value of plotting ===== */
void SetOffset( DRAW *draw )

/*
 *  This function sets offset values necessary for plotting waves. This 
 *  function reads "channels" value and determines offsets from this.
 *  Window heights are also set in this function.
 */

    {
    double   height;     /* height of the frame ( x in LIPS II mode ) */
    int      i;

    height = XMAX - XMIN;
    draw->winheight = height / (double)draw->channels;

    if( draw->channels == 0 ) wprerror( EWPRZEROCH );

    for( i = 0; i < draw->channels; i++ )
	draw->offset[i] = height - draw->winheight * ( (double)i + 0.5 );
    }

/* ===== Rearrange output data ===== */
double *RearrangeData( DRAW *draw, PARAMS *params, double *data )
    {
    double datalen,                 /* time length of the whole data */
           oldsamp, newsamp,        /* old/new sampling rate */
           tiratio,                 /* ratio of old/new time interval */
           datapoint,               /* data calculated from tiratio */
          *newdata;                 /* rearranged data */
    unsigned long oldnod, newnod,   /* old/new number of data */
             i, j,                  /* loop counter */
             intpoint;              /* nearest point in integer */

        /* escaping old data */
    oldsamp = draw->timeint;
    oldnod  = draw->datanum;

    datalen = draw->timeint * (double)draw->datanum;
    newsamp = draw->timescale / DEFAULT_POINTS;

    newnod  = (unsigned long)( datalen / newsamp );
    tiratio = newsamp / oldsamp;

    if( NULL ==
       ( newdata = calloc( newnod * draw->channels, sizeof( double ))))
	wprerror( EWPRNOMEMORY );
    
    for( i = 0; i < draw->channels; i++ ) {
	for( j = 0; j < newnod; j++ ) {
	    datapoint = tiratio * (double)j;
	    intpoint  = roundoff( datapoint );
	    newdata[j + newnod * i] = 
		( data[intpoint + oldnod * i] - draw->aveamp[i] )
	    * ( draw->winheight / 2 / draw->ampscale );
	    }
	}

    /* rearrange DRAW structure */
    draw->timeint = newsamp;
    draw->datanum = newnod;
    SetDrawParams( draw );

    if( params->opts.verbose >= VERBOSE )
	OutputRearrangeParams( draw, oldsamp, newsamp, oldnod, newnod );

    return newdata;
    }

/* ===== Round off datapoint variable ===== */
unsigned long roundoff( double datapoint )
    {
    double  dpfloor,    /* floor( datapoint ) */
            frac;       /* fraction: datapoint - floor( datapoint ) */

    dpfloor = floor( datapoint );
    frac    = datapoint - dpfloor;
    
    if( frac < 0.5 )
	return (unsigned long)dpfloor;
    else
	return (unsigned long)dpfloor + 1;
    }

/* ===== Set drawing pages ===== */
void SetDrawParams( DRAW *draw )

/*
 *  This function is called from SetDraw or RearrangeData, setting parameters
 *  for plotting waves, such as page number and frame points.
 */

    {
    FRPTS  frpts;   /* number of frames and points of the last frame */

/*
 *  Number of pages are set. If number of points are not 2520 * n, more one
 *  page are added.
 */

    draw->pages = (unsigned long)( draw->datanum / draw->points.ord );
    if( draw->datanum % draw->points.ord != 0 )  (draw->pages)++;

       /* plotting points of the last page are set */
    draw->points.last =
	draw->datanum - ( draw->pages - 1 ) * draw->points.ord; 

/*
 *  Calculates number of frames and points of the last frame using function
 *  SetFramesLastPts().
 */

    frpts = SetFrameLastPts( draw->points.ord );
    draw->frames.ord  = frpts.frames;
    draw->lastpts.ord = frpts.lastpts;

    frpts = SetFrameLastPts( draw->points.last );
    draw->frames.last  = frpts.frames;
    draw->lastpts.last = frpts.lastpts;
    }

/* ===== Output rearranged parameters ===== */
void OutputRearrangeParams( DRAW *draw, double oldsamp, double newsamp,
			    unsigned long oldnod, unsigned long newnod )
    {
    fprintf( stderr, "\n===== Parameters Rearranged =====\n" );
    fprintf( stderr, "               OLD --> NEW\n" );
    fprintf( stderr, "Sampling rate: %lf --> %lf\n", oldsamp, newsamp );
    fprintf( stderr, "Number of data: %ld --> %ld\n", oldnod, newnod );
    fprintf( stderr, "===================================\n" );
    }

/* =========================== END of the program ========================== */
