/* ----------------------- Moonquake database utilities ----------------------+
|                                                                             |
|                              "wpr.h" ver. 2.10                              |
|                                                                             |
|                         programmed by Jun-ya Terazono                       |
|      Copyright (c) 1993-1994 by Jun-ya Terazono, all rights reserved.       |
+----------------------------------------------------------------------------*/

/* 
 *  This is a include file for "wpr". This file sets various parameters such
 *  as macro definitions and static variables, and defines structures and 
 *  variable types.
 */

/* ============================= include files ============================= */
#include <ctype.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <mqdb.h>    /* Moonquake database system include file */

/* ========================== Macro definitions ============================ */
    /* general */
#define TRUE    1
#define FALSE   0

    /* file input style */
#define INPUT_ERROR   0      /* file input error */
#define INPUT_STDIN   1      /* standard input */
#define INPUT_FILE    2      /* regular file input */

    /* option functions */
#define WPROPTS    10  /* number of valid options in wpr */
#define SILENT      0  /* no diagnosis message output */
#define VERBOSE     1  /* parameter output only */
#define VERYVERBOSE 2  /* diagnostic information are all output */
#define PAGE        3  /* assigned pages output */
#define HELP        4  /* display help message */
#define AMPSCALE    5  /* change amplitude scale */
#define TIMESCALE   6  /* change time scale */

    /* drawing defaults */
#define DEFAULT_CHANNELS    3     /* number of data channels */
#define FRAME_LENGTH      360     /* maximum drawing points per one frame */
#define DEFAULT_FRAME       7     /* page frames per one page */
#define DEFAULT_POINTS   2520     /* plotting points per one page */
#define DEFAULT_AMPSCALE   10.0   /* amplitude scale */
#define DEFAULT_TIMESCALE 600.0   /* time scale */
#define CLIPRANGE          50     /* range of clipping */

    /* LIPS II+ drawing standard definitions */
#define XMIN    100   /* minimum x coordinate in LIPS II+ mode */
#define XMAX   1750   /* maximum x coordinate in LIPS II+ mode */
#define YMIN    110   /* minimum y coordinate in LIPS II+ mode */
#define YMAX   2630   /* maximum y coordinate in LIPS II+ mode */
#define TIMEBARWIDTH  30    /* width of the time bar */

/* The following definitions are use only for LIPS II+ drawing mode */
#define ESC  0x1b
#define FF   0x0c
#define IS1  0x1F
#define IS2  0x1E

   /* some definitions of control sequence outputs */
#define SoftReset()     printf( "%c<", ESC )      /* soft reset */
#define FullPaintMode() printf( "%c[2&z", ESC );  /* full-paint mode */
#define ShiftToVectorMode()    printf( "%c[&}", ESC );
                                                  /* shift to vector-mode */

    /* for non-ANSI header files */
#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS   0
#endif

#ifndef EXIT_FAILURE
#define EXIT_FAILURE   1
#endif

/* ======================= Structure definitions =========================== */
/*
 *  "opts" is a toggle parameters representing assignation of 
 *  command line parameters.
 */

typedef struct opts {
    int   infile;     /* input file name */
    int   page;       /* assigned page output */
    int   verbose;    /* verbose option ( -v/-V ) */
    int   timescale;  /* time scale option */
    int   ampscale;   /* amplitude scale option */
    }   OPTS;

/*
 *  "params" is a setof parameters assigned by command line options. Meanings
 *  of options are listed in wprerror.c.
 */

typedef struct params {
    OPTS    opts;          /* see above */
    long    pagenum;       /* output page number */
    char    file[BUFSIZ];  /* input data file name */
    double  timescale;     /* time scale */
    double  ampscale;      /* amplitude scale */
    }   PARAMS;

/* 
 *  "fpval" is a set of value used differently in the ordinary page and last
 *  page.
 */

typedef struct fpval   {
    unsigned long   ord;     /* value of ordinary ( not last ) pages */
    unsigned long   last;    /* value of the last page */
    }   FPVAL;

/*
 *  "frpts" is used to deliver values between SetDraw() and SetFrameLastPts().
 */

typedef struct frpts   {
    unsigned long   frames;   /* number of frames */
    unsigned long   lastpts;  /* points of the last frame */
    }   FRPTS;

/*
 *  "draw" is a set of definitions for drawing waves and figure frames by LIPS
 *   II mode. Each member are as follows:
 *       frames:    Number of frames ( ORD / LAST )
 *       lastpts:   Plotting points of the last frame ( ORD / LAST )
 *       points:    Data points of plotting per 1 page ( ORD / LAST )
 *       timeint:   Time interval
 *       channels:  Number of data channels
 *       currch:    Current plotting channel
 *       currpage:  Current output page number
 *       outtime:   Date of output ( by *char )
 *       pages:     Total output pages
 *       timeint:   Time interval of the data 
 *       ampscale:  Amplitude scale
 *       timescale: Plotting time width
 *       samprate:  Sampling rate
 *       winheight: Height of one wave-drawing window
 *       offset:    Offset value of the plot in each channel.
 */

typedef struct draw {
    FPVAL            frames;
    FPVAL            points;
    FPVAL            lastpts;
    unsigned long    channels;
    unsigned long    currch;
    unsigned long    currpage;
    unsigned long    datanum;
    char             outtime[24];
    unsigned long    pages;
    double           ampscale;
    double           timescale;
    double           timeint;
    double           winheight;
    double           offset[MQCHANNELMAX];
    double           aveamp[MQCHANNELMAX];
    }  DRAW;

/*
 *  The following structure is used only in frame drawing. This structure
 *  is used to pack necessary informations for drawing.
 */

typedef struct fparam  {
    unsigned long    DrawFrames;     /* number of drawing frames */
    unsigned short   LastPageFlag;   /* whether current is last page */
    unsigned long    PageStart;      /* start data point of page */
    unsigned long    PageEnd;        /* end data point of page */
    unsigned long    DataStart;
    unsigned long    DataEnd;
    }  FPARAM;

/*
 *   The following three definitions are for representations of LIPS II + 
 *   numerical expressions.
 *
 *   WARNING: The following definitions uses bitfields, namely, very machine
 *   and architecture dependent. If you want to implement this program to other
 *   ( NON-SPARC ) system, please check the byte / word order of CPU, OS and
 *   compilers. If you succeeded in implementing, please send this fix to me.
 */

typedef union lipslong   {
    long      x;
    struct    {
	unsigned int     pad      :4;
	unsigned int     byte5    :6;
	unsigned int     byte4    :6;
	unsigned int     byte3    :6;
	unsigned int     byte2    :6;
	unsigned int     byte1    :4;
	}  bit;
    }    LIPSLONG;

typedef union lipsout    {
    unsigned char        lipsout;
    struct   {
	unsigned int     id       :3;
	unsigned int    sgn       :1;
	unsigned int    byte1     :4;
	}  lowestbyte;
    struct   {
	unsigned int     id       :2;
	unsigned int    byte      :6;
	}  ordbyte;
    }    LIPSOUT;

/* ===================== Static variables definitions ====================== */
static char *optlist[] = 
    { "help", "verbose", "Verbose", "v", "V", "page",
	  "ampscale", "a", "timescale", "t" };
static short optfunc[] = 
    { HELP, VERBOSE, VERYVERBOSE, VERBOSE, VERYVERBOSE, PAGE,
          AMPSCALE, AMPSCALE, TIMESCALE, TIMESCALE };

/* ========================= Function declarations ========================= */
extern void          CheckParams( PARAMS * );
extern void          DisplayUsage( int );
extern void          GetOutputDate( DRAW * );
extern void          GetParams( int, char **, PARAMS * );
extern void          L2BeginPicture( void );
extern void          L2Init( PARAMS * );
extern void          L2DrawEnd( PARAMS * );
extern void          L2DrawFrame( PARAMS * );
extern void          L2DrawStrings( DRAW *, PARAMS *, MqHeader * );
extern void          L2DrawTime( MqTime, DRAW * );
extern void          L2DrawTimeBar( DRAW * );
extern void          L2DrawTimeScale( DRAW * );
extern void          L2DrawWaves( DRAW *, PARAMS *, double * );
extern void          L2PutWaveData( DRAW * );
extern void          OutputDrawParams( DRAW * );
extern void          OutputRearrangeParams( DRAW *, double, double,
                        unsigned long, unsigned long );
extern double *      RearrangeData( DRAW *, PARAMS *, double * );
extern void          SetDefaultParams( PARAMS * );
extern DRAW          SetDraw( PARAMS *, MqHeader * );
extern void          SetDrawParams( DRAW * );
extern FRPTS         SetFrameLastPts( unsigned long );
extern FPARAM        SetFrameParams( PARAMS *, DRAW * );
extern long          SetFramePoints( DRAW *, FPARAM, long );
extern void          SetOffset( DRAW * );
extern double        SetWaveOrigin( long, long, FPARAM, DRAW );
extern void          cal_out( long );
extern void          printout( DRAW *, PARAMS *, MqHeader *, double * );
extern unsigned long roundoff( double );
extern void          wprerror( int );

/* ========================== END of the program =========================== */
